<?php

namespace Feedbii\Router;

/**
 * Class Router
 */
class Router {

    /**
     * @var
     */
    private $url;
    /**
     * @var array
     */
    private $routes = [];
    /**
     * @var array
     */
    private $namedRoutes = [];

    /**
     * @var string
     */
    private $prefix;

    /**
     * Router constructor.
     *
     * @param $url
     */
    public function __construct($url){
        $tmpUrl = explode('?',$url);
        $this->url = $tmpUrl[0];
    }

    /**
     * @param $path
     * @param $callable
     * @param null $name
     *
     * @return Route
     */
    public function get($path, $callable, $name = null){
        return $this->add($path, $callable, $name, 'GET');
    }

    /**
     * @param $path
     * @param $callable
     * @param null $name
     *
     * @return Route
     */
    public function post($path, $callable, $name = null){
        return $this->add($path, $callable, $name, 'POST');
    }

    /**
     * @param $path
     * @param $callable
     * @param $name
     * @param $method
     *
     * @return Route
     */
    private function add($path, $callable, $name, $method){
        $path = $this->prefix.$path;
        $route = new Route($path, $callable);
        $this->routes[$method][] = $route;
        if(is_string($callable) && $name === null){
            $name = $callable;
        }
        if($name){
            $this->namedRoutes[$name] = $route;
        }
        return $route;
    }

    /**
     * @return mixed
     * @throws \Exception
     */
    public function run(){
        if(!isset($this->routes[$_SERVER['REQUEST_METHOD']])){
            throw new \Exception('REQUEST_METHOD does not exist');
        }
        foreach($this->routes[$_SERVER['REQUEST_METHOD']] as $route){
            if($route->match($this->url)){
                return $route->call();
            }
        }
        throw new \Exception('No matching routes');
    }

    /**
     * @param $name
     * @param array $params
     *
     * @return mixed
     * @throws \Exception
     */
    public function url($name, $params = []){
        if(!isset($this->namedRoutes[$name])){
            throw new \Exception('No route matches this name');
        }
        return $this->namedRoutes[$name]->getUrl($params);
    }

    /**
     * @param $prefix
     */
    public function setPrefix($prefix)
    {
        $this->prefix = $prefix;
    }

}